
{*******************************************************}
{                                                       }
{       RichView                                        }
{       Demo: using RVReportHelper to draw header       }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}

{
  Header can be printed automatically, if you assing it using RVPrint.SetHeader
  method. But you cannot use SetHeader if you want to use different headers
  for different pages.
  This demo shows how to use RVReportHelper component for drawing header.
  This is the most complex case: headers have different heights on different
  pages, so heights of areas for the main document are different.

  v1.1
}

unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, Menus, RVStyle, RVScroll, RichView, RVEdit, RVReport, PtblRV,
  Printers;

type
  TForm1 = class(TForm)
    rveMain: TRichViewEdit;
    rvsHeader: TRVStyle;
    MainMenu1: TMainMenu;
    Demo1: TMenuItem;
    Preview1: TMenuItem;
    rvhHeader: TRVReportHelper;
    rvhMain: TRVReportHelper;
    rvsMain: TRVStyle;
    RVPrint1: TRVPrint;
    rvHidden: TRichView;
    Print1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure Preview1Click(Sender: TObject);
    procedure RVPrint1PagePrepaint(Sender: TRVPrint; PageNo: Integer;
      Canvas: TCanvas; Preview: Boolean; PageRect, PrintAreaRect: TRect);
    procedure Print1Click(Sender: TObject);
  private
    { Private declarations }
    procedure MakeHeader(PageNo: Integer);
    procedure FormatDoc(PrnCanvas: TCanvas);
    procedure Preview(PrnCanvas: TCanvas);
    procedure Print;
  public
    { Public declarations }
    CurrentPageNo: Integer;
    PrnCanvas: TCanvas;
  end;

var
  Form1: TForm1;

implementation

uses Unit2;

{$R *.dfm}
{ Form1.OnCreate }
procedure TForm1.FormCreate(Sender: TObject);
begin
  // Creating main document
  rveMain.Clear;
  rveMain.AddNL('Paste here', 0, 0);
  rveMain.Format;

  // rvHidden is a hidden empty TRichView. RVPrint1 "prints" it. But it is empty,
  // so it prints one empty page. All actual printing is performed in
  // RVPrint.OnPagePrepaint
  rvHidden.Visible := False;
  rvHidden.AddNL('', 0, 0);
  RVPrint1.AssignSource(rvHidden);
  RVPrint1.FormatPages(rvdoAll);
end;

{ Returns width and height of printable area on page, minus RVPrint's margins }
procedure GetPageSize(RVPrint: TRVPrint; var Width, Height: Integer);
var DC: HDC;
  phoX, phoY, phW, phH, lpy, lpx, LM, TM, RM, BM: Integer;
begin
  DC := RV_GetPrinterDC; // from PtblRV unit

  Width  := GetDeviceCaps(DC, HORZRES);
  Height := GetDeviceCaps(DC, VERTRES);

  lpy := GetDeviceCaps(DC, LOGPIXELSY);
  lpx := GetDeviceCaps(DC, LOGPIXELSX);

  phoX := GetDeviceCaps(DC, PHYSICALOFFSETX);
  phoY := GetDeviceCaps(DC, PHYSICALOFFSETY);
  phW  := GetDeviceCaps(DC, PHYSICALWIDTH);
  phH  := GetDeviceCaps(DC, PHYSICALHEIGHT);

  LM := MulDiv(RVPrint.LeftMarginMM,   5*lpx, 127)- phoX;
  TM := MulDiv(RVPrint.TopMarginMM,    5*lpy, 127)- phoY;
  RM := MulDiv(RVPrint.RightMarginMM,  5*lpx, 127)- (phW-(phoX+Width));
  BM := MulDiv(RVPrint.BottomMarginMM, 5*lpy, 127)- (phH-(phoY+Height));

  if LM<0 then LM := 0;
  if TM<0 then TM := 0;
  if RM<0 then RM := 0;
  if BM<0 then BM := 0;

  dec(Width, LM+RM);
  dec(Height, TM+BM);

  DeleteDC(DC);

end;

{ Creates header for the PageNo-th page in rvhHeader }
procedure TForm1.MakeHeader(PageNo: Integer);
begin

  rvsHeader.TextStyles.Clear;
  rvsHeader.ParaStyles.Clear;
  rvsHeader.ListStyles.Clear;

  with rvsHeader.TextStyles.Add do begin
    FontName := 'Georgia';
    Size := PageNo*10; // <-- text size depends on PageNo
  end;

  with rvsHeader.ParaStyles.Add do
    Alignment := rvaCenter;

  rvhHeader.RichView.Style := rvsHeader;
  rvhHeader.RichView.Clear;
  rvhHeader.RichView.AddBreak;
  rvhHeader.RichView.AddNL('- '+IntToStr(PageNo)+' - ', 0, 0);
  rvhHeader.RichView.AddBreak;  
end;

{ Prepares document from rveMain for printing in rvhMain }
procedure TForm1.FormatDoc(PrnCanvas: TCanvas);
var PageNo: Integer;
    Stream: TMemoryStream;
    PageWidth, PageHeight: Integer;
begin
  GetPageSize(RVPrint1, PageWidth, PageHeight);

  // Copying from rveMain to rvhMain.RichView
  rvhMain.RichView.Style := rvsMain;
  Stream := TMemoryStream.Create;
  rveMain.SaveRVFToStream(Stream, False);
  Stream.Position := 0;
  rvhMain.RichView.LoadRVFFromStream(Stream);
  Stream.Free;

  // Formatting rvhMain. As you can see, PageHeight parameter of
  // FormatNextPage depends on the header height for this page.
  rvhMain.Init(PrnCanvas, PageWidth);
  PageNo := 1;
  while True do begin
    MakeHeader(PageNo);
    rvhHeader.Init(PrnCanvas, PageWidth);
    rvhHeader.FormatNextPage($FFFFFF);
    if not rvhMain.FormatNextPage(PageHeight-rvhHeader.GetLastPageHeight) then
      break;
    inc(PageNo);
  end;
end;

{ Show Form2 with print preview }
procedure TForm1.Preview(PrnCanvas: TCanvas);
begin
  Form2.ScrollBar1.Position := 1;
  CurrentPageNo := 1;
  Form2.ScrollBar1.Max := rvhMain.PagesCount;
  Form2.RVPrintPreview1.UpdateView;
  Form2.ShowModal;
end;

{ Printing on default printer }
procedure TForm1.Print;
var i: Integer;
begin
  Printer.Title := 'Print Test';
  Printer.BeginDoc;
  for i := 1 to rvhMain.PagesCount do begin
    if i>1 then
      Printer.NewPage;
    CurrentPageNo := i;
    RVPrint1.rv.DrawPage(1, Printer.Canvas, False, False);
  end;
  Printer.EndDoc;
end;

{ Menu: Print preview }
procedure TForm1.Preview1Click(Sender: TObject);
var DC: HDC;
begin
  DC := RV_GetPrinterDC; // from PtblRV unit
  PrnCanvas := TCanvas.Create;
  try
    PrnCanvas.Handle := DC;
    PrnCanvas.Font.PixelsPerInch := GetDeviceCaps(DC, LOGPIXELSY);
    FormatDoc(PrnCanvas);
    Preview(PrnCanvas);
  finally
    PrnCanvas.Handle := 0;
    DeleteDC(DC);
  end;
end;

{ Menu: Print }
procedure TForm1.Print1Click(Sender: TObject);
var DC: HDC;
begin
  DC := RV_GetPrinterDC; // from PtblRV unit
  PrnCanvas := TCanvas.Create;
  try
    PrnCanvas.Handle := DC;
    PrnCanvas.Font.PixelsPerInch := GetDeviceCaps(DC, LOGPIXELSY);
    FormatDoc(PrnCanvas);
    Print;
  finally
    PrnCanvas.Handle := 0;
    DeleteDC(DC);
  end;
end;

{ Printing/ previewing page.
  For RVPrint1, there is only one page (empty page of rvHidden),
  so PageNo is always 1.
  We use the form's variable CurrentPageNo instead. }
procedure TForm1.RVPrint1PagePrepaint(Sender: TRVPrint; PageNo: Integer;
  Canvas: TCanvas; Preview: Boolean; PageRect, PrintAreaRect: TRect);
begin
  MakeHeader(CurrentPageNo);
  rvhHeader.Init(PrnCanvas, PrintAreaRect.Right-PrintAreaRect.Left);
  rvhHeader.FormatNextPage($FFFFFF);
  rvhHeader.DrawPageAt(PrintAreaRect.Left, PrintAreaRect.Top,
    1, Canvas, False, rvhHeader.GetLastPageHeight);
  rvhMain.DrawPageAt(PrintAreaRect.Left, PrintAreaRect.Top+rvhHeader.GetLastPageHeight,
    CurrentPageNo, Canvas, False,
    PrintAreaRect.Bottom-PrintAreaRect.Top-rvhHeader.GetLastPageHeight);
end;

end.

